/***********************************************************
SPLINE.PRG

This is a GAUSS program, written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen

This program estimates the spline transfer function model presented in
"How Responsive are Private Transfers to Income?
Evidence From a Laissez-Faire Economy"
by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

The results of this program generates the estimates reported in Table 3
of the paper, and Figures 2 and 3.

This program will take a long time to execute if one sets qn=0.  To speed up
execution (with little cost in precision), set qn=500 or even qn=100.

***********************************************************/

trm = .02;            @ Percentile of high-income households deleted  @
_per = .10;           @ Percentage to trim from quantiles for threshold search @
qn   = 0;             @ Set qn=0  to search across all thresholds.
                        Set qn to a positive integers to search
                        across a fixed number of quantiles     @
_autop = 0;           @ If set to 1, will automatically print graphs   @
_noinc=1;             @ If set to 1, includes dummy for no income, else set to 0 @
_quad=0;              @ If set to 1, includes quadratic term for non-altruistic regime @
_age2=0;               @ If set to 1, includes quadratic term in age of household head @
_save=0;               @ Set to 1 to save transfer functions @

/***********************************************************/
output file = spline.out reset;
output off;
/***********************************************************/

smp=1; do while smp<=2;              @ Set to 1 if "urban" or 2 if "rural"  @

let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
  rname ="Region0"|rname;
  sample="Urban";
elseif smp==2;
  sample="Rural";
endif;

dfile = "" $+ sample $+ ".dat";
vn = getname(dfile);
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";

y = abroad+domestic+inkind-gifts;
i = totinc+gifts-abroad-domestic-inkind-retinc;

dat=delif(dat~i~y,(i.<0));         @ delete observations with negative income @

dat = sortc(dat,28);
n = floor(rows(dat)*(1-trm));
dat = dat[1:n,.];
retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];
dat= dat[.,1:27];


dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
vn[23]="Adults";
xx=(retinc.>0)~retinc~dat[.,12:17];
xn="Has Retr"|vn[10 12:17];
if _age2==1;
  xx=xx~(dat[.,17].^2);
  xn=xn|"Age^2";
endif;
region=dat[.,27];
r=unique(region,1);
xreg=(region.==(r'));
xx=xx~dat[.,18:26]~xreg;
xn=xn|vn[18:26]|rname;
mx = ones(n,1)*(meanc(xx)');
if _noinc==1;
  i0=(i.==0);
  xx=i0~xx;
  xn="No Income"|xn;
endif;

output on;
"************************************************************";
"";"";
sh = sample $+ " households";
$sh;
print ("Dependent Variable:     " $+ yname);
"Trimmed Top Quantile of Income " trm;
if _noinc==1;
"Number of Zero Income          " sumc(i0);
endif;
"Observations                      " n;
"__________________________________________________";
output off;


/*************************************************/
@ Linear Model @
x=i~xx;
beta = y/x;
e = y - x*beta;
xname = "Income"|xn;

output on;
  "Linear Model";
  call report(y,x,e,beta,xname);
output off;

/*************************************************/
@ Polynomial Model @
p=9;
xp = (i.^(seqa(1,1,p)'))./(10000.^(seqa(0,1,p)'));
x=xp~xx;
beta = y/x;
e = y - x*beta;
trp = xp*beta[1:p]+mx*beta[p+1+_noinc:rows(beta)];

xname = "Income";
jp=2; do while jp<=p;
  xjp="" $+ "Income^" $+ ftocv(jp,1,0);
  xname=xname|xjp;
jp=jp+1;endo;
xname = xname|xn;

output on;
  "Polynomial Model";
  call report(y,x,e,beta,xname);
output off;

/*************************************************/

xm = xx*invpd(moment(xx,0));
e0 = y - xm*(xx'y);
ee0 = e0'e0;
q=i;
if qn == 0;
  q = unique(q[floor(_per*n):n-floor(_per*n)],1);
  qn = rows(q);
else;
  q = q[floor(seqa(_per,(1-_per*2)/qn,qn)*n)];
endif;

/*************************************************/
@ Linear Spline Model @

sse = zeros(qn,1);
j=1;do while j<=qn;
  qj = q[j];
  ic = i - qj;
  i1 = (i.<=qj);
  i2 = 1-i1;
  dx = (ic.*i1)~(ic.*i2);
  if _quad==1;
    dx=dx~(((ic.^2).*i2)/(10^6));
  endif;
  xv = dx - xm*(xx'dx);
  et = e0 - xv*(e0/xv);
  sse[j] = et'et;
  if round(j/10)==(j/10);
    j~qj~(sse[j]/n);
  endif;
j=j+1;endo;
jmin = minindc(sse);
knot = q[jmin];

ic = i - knot;
i1 = (i.<=knot);
i2 = 1-i1;
dx = (ic.*i1)~(ic.*i2);
if _quad==1;
  dx=dx~(((ic.^2).*i2)/(10^6));
endif;
x  = dx~xx;
beta = y/x;
e = y - x*beta;
xh = -beta[1].*i1-beta[2].*i2;
if _quad==1;
  xh = xh-(ic.*i2)*(2*beta[3]);
endif;
x=x~xh;
tr1 = dx*beta[1:2+_quad]+mx*beta[3+_quad+_noinc:rows(beta)];

xname = "Slope_1"|"Slope_2";
if _quad==1;
  xname=xname|"Quad_2";
endif;
xname=xname|xn|"Threshold";
output on;
  if _quad==0;
    "Linear Spline Model";
  elseif _quad==1;
    "Linear/Quadratic Spline Model";
  endif;
  "# Quantiles Used   " qn;
  "Income Threshold Quantile :" meanc(i.<=knot)';
  call report(y,x,e,beta|knot,xname);
output off;

call gr_tr(i,trp~tr1);

/*************************************************/

if _save==1;
  tr=i~trp~tr1;
  trname = "" $+ "trans_" $+ ftocv(smp,1,0) $+ ".fmt";
  save ^trname=tr;
endif;

smp = smp+1; endo;

/*************************************************/

proc (0) = gr_tr(i,tr);
local sh2,pn,zp;
graphset;
_plwidth=2;
_pdate="";
if _autop==1;
 graphprt("-p -pr=3");
endif;
fonts("complex simgrma");
sh2 = sample $+ " households";
title(sh2);
ylabel("Transfers (Pesos)");
xlabel("Total Income (Pesos)");
_pxsci = 6; _pysci = 6;
if smp==1;
 xtics(0,200000,50000,5); @300000@
 ytics(0,20000,5000,5); @0 to 20000@
elseif smp==2;
 xtics(0,80000,20000,4);
 ytics(0,10000,2000,4);
endif;
xy(i,tr);
endp;

proc (0) = report(y,x,e,theta,xname);
local ee,sig,mi,se,seh,pr,ff;
  ee = e'e;
  sig=ee/(rows(y)-cols(x));
  mi = invpd(moment(x,0));
  se = sqrt(abs(diag(mi*sig)));
  seh=sqrt(abs(diag(mi*moment(x.*e,0)*mi)));
  "Sum of Squared Errors               " ee;
  "Residual Standard Deviation         " sqrt(sig);
  "R-squared                           " (1 - ee/(sumc((y-meanc(y)).^2)));
  "";
  ff = "#*.*lG";
  pr = printfm("Variable"~"Estimate"~"St Error"~"Het St Er",0~0~0~0,ff~16~8);"";
  "       ____________________________________________________";
  pr = printfm(xname~theta~se~seh,0~1~1~1,ff~16~8);"";
  "__________________________________________________";
  "";"";"";
endp;

/*************************************************/
