/* SMOOTH.PRG

This is a GAUSS program, written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen

This program estimates the spline transfer function model presented in
"Are Private Transfers Altruistically Motivated?",
by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

This program estimates the model with random thresholds.

*/

trm = .02;            @ Percentile of high-income households deleted from sample
 @
_per = .05;           @ Percentage to trim from quantiles for threshold search @
qn   = 100;             @ Set qn=0  to search across all thresholds.
                        Set qn to a positive integers to search
                        across a fixed number of quantiles     @
_autop = 0;           @ If set to 1, will automatically print graphs   @

output file = smooth.out reset;
output off;
optset;

/***********************************************************/

smp=1; do while smp<=2;              @ Set to 1 if "urban" or 2 if "rural"  @

let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
  rname ="Region0"|rname;
  sample="Urban";
elseif smp==2;
  sample="Rural";
endif;

dfile = "" $+ sample $+ ".dat";
vn = getname(dfile);
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";

y = abroad+domestic+inkind-gifts;
i = totinc+gifts-abroad-domestic-inkind-retinc;

dat=delif(dat~i~y,(i.<0));         @ delete observations with negative income @

dat = sortc(dat,28);
n = floor(rows(dat)*(1-trm));
dat = dat[1:n,.];
retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];
dat= dat[.,1:27];


dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
vn[23]="Adults";
xx=(retinc.>0)~retinc~dat[.,12:17];
xn="Has Retr"|vn[10 12:17];
region=dat[.,27];
r=unique(region,1);
xreg=(region.==(r'));
xx=xx~dat[.,18:26]~xreg;
xn=xn|vn[18:26]|rname;
mx = ones(n,1)*(meanc(xx)');
i0=(i.==0);
xx=i0~xx;
xn="No Income"|xn;

output on;
"************************************************************";
"";"";
print ("Dependent Variable:     " $+ yname);
sh = sample $+ " households";
$sh;
"Trimmed Top Quantile of Income " trm;
"Number of Zero Income          " sumc(i0);
"Observations                      " n;
"__________________________________________________";
output off;


/*************************************************/
xm = xx*invpd(moment(xx,0));
e0 = y - xm*(xx'y);
ee0 = e0'e0;
if qn == 0;
  q = sortc(i,1);
  q = unique(q[floor(_per*n):n-floor(_per*n)],1);
  qn = rows(q);
else;
  q = sortc(i,1);
  q = q[floor(seqa(_per,(1-_per*2)/qn,qn)*n)];
endif;

/*************************************************/
@ Continuous Linear Spline Model (for starting values) @
sse = zeros(qn,1);
j=1;do while j<=qn;
  qj = q[j];
  ic = i - qj;
  i1 = (i.<=qj);
  i2 = 1-i1;
  dx = (ic.*i1)~(ic.*i2);
  xv = dx - xm*(xx'dx);
  et = e0 - xv*(e0/xv);
  sse[j] = et'et;
  if round(j/10)==(j/10);
    j~qj~(sse[j]/n);
  endif;
j=j+1;endo;
jmin = minindc(sse);
qhat = q[jmin];

ic = i - qhat;
i1 = (i.<=qhat);
i2 = 1-i1;
dx = (ic.*i1)~(ic.*i2);
x  = dx~xx;
beta = y/x;
e = y - x*beta;
ee = e'e;

ba=beta[3:rows(beta)];
ya = y - xx*ba;
tr1 = dx*beta[1 2]+mx*beta[4:rows(beta)];


/*************************************************/


@ Smooth Model - Step 1 @

proc ss(theta);
local ik,omega,a,cdfa,dx,e,s;
  ik=i-theta[1];
  omega=abs(theta[2]);
  a=ik./omega;
  cdfa=cdfn(a);
  dx=ik.*((1-cdfa)~cdfa)+(pdfn(a)*((-1)~1))*omega;
  e=ya-dx*(ya/dx);
  s=(e'e-ee)/rows(e);
retp(s);
endp;

theta0=((qhat|1000)~((qhat-2000)|1000)~(qhat|5000)~((qhat-2000)|8000));
tn=cols(theta0);
theta=zeros(2,tn);
smin=zeros(tn,1);

_opalgr=5;
ti=1; do while ti<=tn;
  {theta[.,ti],smin[ti],gr,ret}=optmum(&ss,theta0[.,ti]);
ti=ti+1;endo;
theta1=theta[.,minindc(smin)];

@ Smooth Model - Step 2 @

proc s_smooth(theta);
local ik,omega,a,cdfa,dx,xv,e,s;
  ik=i-theta[1];
  omega=abs(theta[2]);
  a=ik./omega;
  cdfa=cdfn(a);
  dx=ik.*((1-cdfa)~cdfa)+(pdfn(a)*((-1)~1))*omega;
  xv = dx - xm*(xx'dx);
  e  = e0 - xv*(e0/xv);
  s  = (e'e-ee)/rows(e);
retp(s);
endp;

{theta,smin,gr,ret}=optmum(&s_smooth,theta1);

knot=theta[1];
omega=abs(theta[2]);
ik=i-knot;
a=ik./omega;
cdfa=cdfn(a);
pdfa=pdfn(a);
dx=ik.*((1-cdfa)~cdfa)+(pdfa*((-1)~1))*omega;
x  = dx~xx;
beta = y/x;
e = y - x*beta;
ee = e'e;
k = cols(x);
sig = (e'e)/(n-k-2);
r_2 = 1 - ee/(sumc((y-meanc(y)).^2));
theta=knot|omega|beta;

if omega>10;
b1=beta[1];b2=beta[2];b12=b1-b2;
g1=-((1-cdfa)*b1+cdfa*b2)/omega+a.*pdfa*b12*(1-omega)/omega;
g2=-((1-cdfa)*b1+cdfa*b2).*a/omega-pdfa*b12+pdfa.*(a.^2).*(b12*(1-omega)/omega);
gx=g1~g2~x;
h=invpd(moment(gx,0));
se=sqrt(diag(h)*sig);
seh=sqrt(diag(h*moment(gx.*e,0)*h));
else; se=zeros(rows(theta),1);seh=se;endif;

ff = "#*.*lG";
xname = "Knot"|"Omega"|"Slope_1"|"Slope_2"|xn;
output on;
"Smooth Spline Model";
"Sum of Squared Errors               " ee;
"Residual Standard Deviation         " sqrt(sig);
"R-squared                           " r_2;
"";
pr = printfm("Variable"~"Estimate"~"St Error"~"Het St Er",0~0~0~0,ff~16~8);"";
"       ____________________________________________________";
pr = printfm(xname~theta~se~seh,0~1~1~1,ff~16~8);"";
"__________________________________________________";
"";"";"";
output off;

mx = ones(n,1)*(meanc(xx)');
xr = dx~mx;
tr3=xr*beta;
call gr_tr(i,tr1~tr3);

smp = smp+1; endo;


/*************************************************/

proc (0) = gr_tr(i,tr);
local sh2,pn,zp;
graphset;
_plwidth=2;
_pdate="";
if _autop==1;
 graphprt("-p -pr=3");
endif;
fonts("complex simgrma");
sh2 = sample $+ " households";
title(sh2);
ylabel("Transfers (Pesos)");
xlabel("Total Income (Pesos)");
_pxsci = 6; _pysci = 6;
if smp==1;
 xtics(0,200000,50000,5); @300000@
 ytics(0000,20000,5000,5); @0 to 20000@
elseif smp==2;
 xtics(0,80000,20000,4);
 ytics(0,10000,2000,4);
endif;
xy(i,tr);
endp;

/*************************************************/
